package rofaapp;

import javafx.scene.control.Spinner;
import sun.applet.Main;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class AddItemWindow extends JDialog implements ActionListener
{
    private ItemOrder order;

    private TypeWood woodTypes[];
    
    private boolean isEdit = false;
    private int editIndex = -1;


    //Main Panels
    JPanel chairPanel;
    JPanel tablePanel;
    JPanel deskPanel;
    JPanel yesnoPanel;

    //Swicher Buttons
    JButton buttonSwitchChair;
    JButton buttonSwitchTable;
    JButton buttonSwitchDesk;

    //YesNo Buttons
    JButton yesButton;
    JButton cancelButton;

    //Spinners
    JSpinner idSpinnerChair;
    JSpinner quantitySpinnerChair;
    JSpinner idSpinnerDesk;
    JSpinner drawsSpinnerDesk;
    JSpinner widthSpinnerDesk;
    JSpinner depthSpinnerDesk;
    JSpinner idSpinnerTable;
    JSpinner quantitySpinnerTable;
    JSpinner diameterSpinnerTable;

    //Combo Boxes
    JComboBox woodComboBoxChair;
    JComboBox woodComboBoxDesk;
    JComboBox woodComboBoxTable;

    //Check Boxes
    JCheckBox armrestsCheckBoxChair;
    JCheckBox chromeCheckBoxTable;


    /**
     * Constructor used when adding a new item.
     * <br>
     * Calls all initialisation functions for the window: {@link #initWindowSettings()}, {@link #initSelectButtons()}, {@link #initSelectedPanels()}, {@link #initYesNoButtons()}.
     * @param defaultPanel Selects the panel that will be shown when the window is first shown (the default panel). Calls/is used in the function {@link #changeSelectedPanel(int)}.
     * @param o The current item order that will be edited.
     * @param tw An array if all of the wood types.
     */
    public AddItemWindow(int defaultPanel, ItemOrder o, TypeWood[] tw)
    {
        order = o;
        woodTypes = tw;

        //This constructor is called when adding a new item.
        //Init functions.
        initWindowSettings();
        initSelectButtons();
        initSelectedPanels();
        initYesNoButtons();
        //Change panel to set default.
        changeSelectedPanel(defaultPanel);
        //Set the window location relative to null (center screen).
        setLocationRelativeTo(null);
        //Set the window to visible after all init has been done.
        setVisible(true);
    }
    
    /**
     * Constructor used when editing an existing item.
     * Calls all initialisation functions for the window: {@link #initWindowSettings()}, {@link #initSelectButtons()}, {@link #initSelectedPanels()}, {@link #initYesNoButtons()}.
     * <br>
     * For the edit constructor, calls {@link #loadEditSettings(Furniture)} which shows the current item's attributes in the window.
     * @param defaultPanel Selects the panel that will be shown when the window is first shown (the default panel). Calls/is used in the function {@link #changeSelectedPanel(int)}.
     * @param o The current item order that will be edited.
     * @param tw An array if all of the wood types.
     * @param index The index of the currently edited item within the ItemOrder (0-8, if not, will throw an exception).
     * @param f The furniture object of the item that is being edited.
     */
    public AddItemWindow(int defaultPanel, ItemOrder o, TypeWood[] tw, int index, Furniture f)
    {
        order = o;
        woodTypes = tw;
        editIndex = index;

        //This constructor is called when editing an item.
        initWindowSettings();
        initSelectButtons();
        initSelectedPanels();
        initYesNoButtons();
        loadEditSettings(f);
        changeSelectedPanel(defaultPanel);
        setLocationRelativeTo(null);
        setVisible(true);
    }
    
    /**
     * Initialises basic elements of the window (such as layout, the default close operation and the modality type).
     */
    private void initWindowSettings()
    {
        //Needed to make the main thread wait until this dialog is disposed
        setModalityType(ModalityType.APPLICATION_MODAL);
        setResizable(false);
        setLayout(new FlowLayout());
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        setTitle("Add Item");
    }
    
    /**
     * Initialises the three buttons displayed at the top of the frame (instantiates them, adds their text, creates the actionlistener and adds them to the panel).
     */
    private void initSelectButtons()
    {
        JPanel buttonSwitcher = new JPanel();
        buttonSwitcher.setPreferredSize(new Dimension(500, 50));
        buttonSwitcher.setVisible(true);
        this.add(buttonSwitcher);

        buttonSwitchChair = new JButton("Chair");
        buttonSwitchTable = new JButton("Table");
        buttonSwitchDesk = new JButton("Desk");

        buttonSwitchChair.addActionListener(this);
        buttonSwitchDesk.addActionListener(this);
        buttonSwitchTable.addActionListener(this);

        buttonSwitchChair.setPreferredSize(new Dimension(150, 30));
        buttonSwitchTable.setPreferredSize(new Dimension(150, 30));
        buttonSwitchDesk.setPreferredSize(new Dimension(150, 30));

        buttonSwitcher.add(buttonSwitchChair);
        buttonSwitcher.add(buttonSwitchTable);
        buttonSwitcher.add(buttonSwitchDesk);
    }

    /**
     * Initialises the 'Yes-No' buttons shown at the bottom of the form (creates them and adds them to the panel).
     */
    private void initYesNoButtons()
    {
        yesButton = new JButton("Confirm");
        cancelButton = new JButton("Cancel");
        yesnoPanel = new JPanel();
        yesButton.addActionListener(this);
        cancelButton.addActionListener(this);
        yesnoPanel.add(yesButton);
        yesnoPanel.add(cancelButton);
    }

    
    /**
     * Creates all of the panels and buttons and elements needed for each of the panels for each of the furniture items. Creates all elements, adds the action listeners for them and adds them to the window. (WARNING: Very long function of repetitiveness).
     */
    private void initSelectedPanels()
    {
        //Main Panels
        chairPanel = new JPanel();
        tablePanel = new JPanel();
        deskPanel = new JPanel();

        //Sub Panels
        //Chair
        JPanel idPanelC = new JPanel();
        JPanel quantityPanelC = new JPanel();
        JPanel woodPanelC = new JPanel();
        JPanel armPanelC = new JPanel();
        //Table
        JPanel idPanelT = new JPanel();
        JPanel quantityPanelT = new JPanel();
        JPanel woodPanelT = new JPanel();
        JPanel chromePanelT = new JPanel();
        JPanel diameterPanelT = new JPanel();
        //Desk
        JPanel idPanelD = new JPanel();
        JPanel quantityPanelD = new JPanel();
        JPanel woodPanelD = new JPanel();
        JPanel drawsPanelD = new JPanel();
        JPanel heightPanelD = new JPanel();
        JPanel widthPanelD = new JPanel();
        JPanel depthPanelD = new JPanel();

        chairPanel.setPreferredSize(new Dimension(500, 140));
        tablePanel.setPreferredSize(new Dimension(500, 175));
        deskPanel.setPreferredSize(new Dimension(500, 250));

        //Set Sizes for Sub Panels
        Dimension subPanelDimension = new Dimension(500, 30);
        idPanelC.setPreferredSize(subPanelDimension);
        woodPanelC.setPreferredSize(subPanelDimension);
        armPanelC.setPreferredSize(subPanelDimension);
        quantityPanelC.setPreferredSize(subPanelDimension);

        idPanelT.setPreferredSize(subPanelDimension);
        quantityPanelT.setPreferredSize(subPanelDimension);
        woodPanelT.setPreferredSize(subPanelDimension);
        chromePanelT.setPreferredSize(subPanelDimension);

        idPanelD.setPreferredSize(subPanelDimension);
        quantityPanelD.setPreferredSize(subPanelDimension);
        woodPanelD.setPreferredSize(subPanelDimension);
        drawsPanelD.setPreferredSize(subPanelDimension);
        heightPanelD.setPreferredSize(subPanelDimension);
        widthPanelD.setPreferredSize(subPanelDimension);
        depthPanelD.setPreferredSize(subPanelDimension);

        //Sub Elements

        ///////////////
        //Chair Panel//
        ///////////////

        //ID
        JLabel idLabelChair = new JLabel("                    ID:");
        idSpinnerChair = new JSpinner(new SpinnerNumberModel(order.getFirstAvailable() + 1, 1, 9999, 1));
        idSpinnerChair.setPreferredSize(new Dimension(200, 25));
        //Quantity
        JLabel quantityLabelChair = new JLabel("        Quantity:");
        quantitySpinnerChair = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));
        quantitySpinnerChair.setPreferredSize(new Dimension(200, 25));
        //Wood Type
        JLabel woodLabelChair = new JLabel("  Wood Type:");
        woodComboBoxChair = new JComboBox(woodTypes);//GET WOOD TYPES
        woodComboBoxChair.setPreferredSize(new Dimension(200, 25));
        //Armrests
        JLabel armrestsLabelChair = new JLabel("   Armrests:");
        armrestsCheckBoxChair = new JCheckBox();
        armrestsCheckBoxChair.setPreferredSize(new Dimension(200, 25));
        

        ///////////////
        //Table Panel//
        ///////////////

        //ID
        JLabel idLabelTable = new JLabel("                 ID:");
        idSpinnerTable = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));
        idSpinnerTable.setPreferredSize(new Dimension(200, 25));
        //Quantity
        JLabel quantityLabelTable = new JLabel("     Quantity:");
        quantitySpinnerTable = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));
        quantitySpinnerTable.setPreferredSize(new Dimension(200, 25));
        //Wood Type
        JLabel woodLabelTable = new JLabel("Wood Type:");
        woodComboBoxTable = new JComboBox(woodTypes);//GET WOOD TYPES
        woodComboBoxTable.setPreferredSize(new Dimension(200, 25));
        //Chrome
        JLabel chromeLabelTable = new JLabel("   Chrome:");
        chromeCheckBoxTable = new JCheckBox();
        chromeCheckBoxTable.setPreferredSize(new Dimension(200, 25));
        //Diameter
        JLabel diameterLabelTable = new JLabel("   Diameter:");
        diameterSpinnerTable = new JSpinner(new SpinnerNumberModel(50, 50, 9999, 1));
        diameterSpinnerTable.setPreferredSize(new Dimension(200, 25));

        //////////////
        //Desk Panel//
        //////////////

        //ID
        JLabel idLabelDesk = new JLabel("                 ID:");
        idSpinnerDesk = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));//CHANGE THIS ONE!!!!!!
        idSpinnerDesk.setPreferredSize(new Dimension(200, 25));
        //Quantity
        JLabel quantityLabelDesk = new JLabel("     Quantity:");
        JSpinner quantitySpinnerDesk = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));
        quantitySpinnerDesk.setPreferredSize(new Dimension(200, 25));
        //Wood Type
        JLabel woodLabelDesk = new JLabel("Wood Type:");
        woodComboBoxDesk = new JComboBox(woodTypes);//GET WOOD TYPES
        woodComboBoxDesk.setPreferredSize(new Dimension(200, 25));
        //Draws
        JLabel drawsLabelDesk = new JLabel("        Draws:");
        drawsSpinnerDesk = new JSpinner(new SpinnerNumberModel(1, 1, 4, 1));
        drawsSpinnerDesk.setPreferredSize(new Dimension(200, 25));
        //Height
        JLabel heightLabelDesk = new JLabel("        Height:");
        JSpinner heightSpinnerDesk = new JSpinner(new SpinnerNumberModel(80, 80, 80, 1));
        heightSpinnerDesk.setPreferredSize(new Dimension(200, 25));
        heightSpinnerDesk.setEnabled(false);
        //Width
        JLabel widthLabelDesk = new JLabel("         Width:");
        widthSpinnerDesk = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));
        widthSpinnerDesk.setPreferredSize(new Dimension(200, 25));
        //Depth
        JLabel depthLabelDesk = new JLabel("         Depth:");
        depthSpinnerDesk = new JSpinner(new SpinnerNumberModel(1, 1, 9999, 1));
        depthSpinnerDesk.setPreferredSize(new Dimension(200, 25));

        //Add Elements to Panels

        ///////////////
        //Chair Panel//    }
        ///////////////

        //ID
        idPanelC.add(idLabelChair);
        idPanelC.add(idSpinnerChair);
        //QUANTITY
        quantityPanelC.add(quantityLabelChair);
        quantityPanelC.add(quantitySpinnerChair);
        //WOODTYPE
        woodPanelC.add(woodLabelChair);
        woodPanelC.add(woodComboBoxChair);
        //ARMRESTS
        armPanelC.add(armrestsLabelChair);
        armPanelC.add(armrestsCheckBoxChair);

        ///////////////
        //Table Panel//
        ///////////////

        //ID
        idPanelT.add(idLabelTable);
        idPanelT.add(idSpinnerTable);
        //QUANTITY
        quantityPanelT.add(quantityLabelTable);
        quantityPanelT.add(quantitySpinnerTable);
        //WOOD TYPE
        woodPanelT.add(woodLabelTable);
        woodPanelT.add(woodComboBoxTable);
        //CHROME
        chromePanelT.add(chromeLabelTable);
        chromePanelT.add(chromeCheckBoxTable);
        //DIAMETER
        diameterPanelT.add(diameterLabelTable);
        diameterPanelT.add(diameterSpinnerTable);

        //////////////
        //Desk Panel//
        //////////////

        //ID
        idPanelD.add(idLabelDesk);
        idPanelD.add(idSpinnerDesk);
        //QUANTITY
        quantityPanelD.add(quantityLabelDesk);
        quantityPanelD.add(quantitySpinnerDesk);
        //WOOD TYPE
        woodPanelD.add(woodLabelDesk);
        woodPanelD.add(woodComboBoxDesk);
        //DRAWS
        drawsPanelD.add(drawsLabelDesk);
        drawsPanelD.add(drawsSpinnerDesk);
        //HEIGHT
        heightPanelD.add(heightLabelDesk);
        heightPanelD.add(heightSpinnerDesk);
        //WIDTH
        widthPanelD.add(widthLabelDesk);
        widthPanelD.add(widthSpinnerDesk);
        //DEPTH
        depthPanelD.add(depthLabelDesk);
        depthPanelD.add(depthSpinnerDesk);

        //Chair Panel
        chairPanel.add(idPanelC);
        chairPanel.add(quantityPanelC);
        chairPanel.add(woodPanelC);
        chairPanel.add(armPanelC);
        //Table Panel
        tablePanel.add(idPanelT);
        tablePanel.add(quantityPanelT);
        tablePanel.add(woodPanelT);
        tablePanel.add(chromePanelT);
        tablePanel.add(diameterPanelT);
        //Desk Panel
        deskPanel.add(idPanelD);
        deskPanel.add(quantityPanelD);
        deskPanel.add(woodPanelD);
        deskPanel.add(drawsPanelD);
        deskPanel.add(heightPanelD);
        deskPanel.add(widthPanelD);
        deskPanel.add(depthPanelD);
    }
    
    /**
     * Loads a given furniture item, and changes the window elements to the attributes of the given furniture item.
     * @param f Given furniture item.
     */
    private void loadEditSettings(Furniture f)
    {
        //Set flag to show this instance is an edit.
        isEdit = true;
        if (f instanceof ItemTable)
        {
            //Cast to ItemTable
            ItemTable it = (ItemTable)f;
            //Set values to edited object
            idSpinnerTable.setValue(it.getId());
            woodComboBoxTable.setSelectedItem(it.getWood());
            chromeCheckBoxTable.setSelected(f instanceof ItemTableChrome);
            diameterSpinnerTable.setValue(it.getDiameter());
        }
        else if (f instanceof ItemChair)
        {
            //Cast to ItemChair
            ItemChair ic = (ItemChair)f;
            idSpinnerChair.setValue(ic.getId());
            woodComboBoxChair.setSelectedItem(ic.getWood());
            armrestsCheckBoxChair.setSelected(f instanceof ItemArmChair);
        }
        else if (f instanceof ItemDesk)
        {
            //Cast to ItemDesk
            ItemDesk id = (ItemDesk)f;
            idSpinnerDesk.setValue(id.getId());
            woodComboBoxDesk.setSelectedItem(id.getWood());
            drawsSpinnerDesk.setValue(id.getDraws());
            widthSpinnerDesk.setValue(id.getWidth());
            depthSpinnerDesk.setValue(id.getDepth());
        }
        setTitle("Edit Item: " + f.getName());
    }
    
    /**
     * Simple method for giving an instance of furnature and getting back the tab index of the object in the window.
     * @param f The furniture item.
     * @return The tab index of the given item.
     */
    public static int getTabValue(Furniture f)
    {
        if (f instanceof ItemChair)
            return 1;
        else if (f instanceof ItemDesk)
            return 2;
        else if (f instanceof ItemTable)
            return 3;
        else
            return -1;
    }
    
    /**
     * Changes the currently selected tab.
     * @param i The tab index to switch views to. (1-3. If not a valid index, will display an empty panel.)
     */
    private void changeSelectedPanel(int i)
    {
        remove(yesnoPanel);
        if (i == 1)
        {
            remove(deskPanel);
            remove(tablePanel);
            add(chairPanel);
            //Validate needed for repaint
            validate();
            repaint();
            setSize(490, 290);
        }
        else if (i == 2)
        {
            remove(tablePanel);
            remove(chairPanel);
            add(deskPanel);
            //Validate needed for repaint
            validate();
            repaint();
            setSize(new Dimension(490, 410));
        }
        else if (i == 3)
        {
            remove(chairPanel);
            remove(deskPanel);
            add(tablePanel);
            //Validate needed for repaint
            validate();
            repaint();
            setSize(new Dimension(490, 330));
        }
        add(yesnoPanel);
    }

    /**
     * @return The current order within the window instance.
     */
    public ItemOrder getUpdatedOrder()
    {
        return order;
    }
    
    /**
     * Action listener for the buttons shown on the window.
     * @param e ActionEvent.
     */
    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource().equals(buttonSwitchChair) && chairPanel.getParent() == null)
        {
            changeSelectedPanel(1);
        }
        else if (e.getSource().equals(buttonSwitchTable) && tablePanel.getParent() == null)
        {
            changeSelectedPanel(3);
        }
        else if (e.getSource().equals(buttonSwitchDesk) && deskPanel.getParent() == null)
        {
            changeSelectedPanel(2);
        }
        else if (e.getSource().equals(cancelButton))
        {
            dispose();
        }
        else if (e.getSource().equals(yesButton))
        {
            if (order.getFreeSpace() != 0)
            {
                Furniture ic = null;
                if (deskPanel.getParent() != null)
                {
                    //Desk Panel is currently selected (displayed)
                    ic = new ItemDesk((int)idSpinnerDesk.getValue(), (TypeWood)woodComboBoxDesk.getSelectedItem(), ItemIcon.DESK_ICON[(int)drawsSpinnerDesk.getValue()], (int)drawsSpinnerDesk.getValue(), (int)widthSpinnerDesk.getValue(), (int)depthSpinnerDesk.getValue());
                }
                else if (tablePanel.getParent() != null)
                {
                    if (chromeCheckBoxTable.isSelected())
                    {
                        ic = new ItemTableChrome((int)idSpinnerTable.getValue(), (TypeWood)woodComboBoxTable.getSelectedItem(), ItemIcon.TABLE_ICON_1, (int)diameterSpinnerTable.getValue());
                    }
                    else
                    {
                        ic = new ItemTableWood((int)idSpinnerTable.getValue(), (TypeWood)woodComboBoxTable.getSelectedItem(), ItemIcon.TABLE_ICON_2, (int)diameterSpinnerTable.getValue());
                    }
                    //Table Panel is currently selected (displayed)
                }
                else if (chairPanel.getParent() != null)
                {
                    //Chair Panel is currently selected (displayed)
                    if (armrestsCheckBoxChair.isSelected())
                    {
                        ic = new ItemArmChair((int)idSpinnerChair.getValue(), (TypeWood)woodComboBoxChair.getSelectedItem(), ItemIcon.CHAIR_ICON_2);
                    }
                    else
                    {
                        ic = new ItemChair((int)idSpinnerChair.getValue(), (TypeWood)woodComboBoxChair.getSelectedItem(), ItemIcon.CHAIR_ICON_1);
                    }
                }
                if (isEdit)
                {
                    order.add(ic, editIndex, true);
                }
                else
                {
                    order.add(ic, order.getFirstAvailable(), true);
                }
                
                dispose();
            }
            else
            {
                JOptionPane.showMessageDialog(this, "There is no space left in this order for any other items.", "Order Full!", JOptionPane.INFORMATION_MESSAGE);
            }
        }
    }
}
