package rofaapp;

import java.io.Serializable;
import java.lang.reflect.Type;

public class ItemOrder implements Serializable{

    private final transient int arraySize = 9;
    private int currentSize;
    private Furniture orderList[];

    /**
     * Creates an empty item order of {@link #arraySize} items.
     */
    ItemOrder()
    {
        orderList = new Furniture[arraySize];
        for (int i = 0; i < arraySize; i++)
        {
            //NULL EVERYTHING!!!!!!!!!!!!!!!!
            orderList[i] = null;
        }
    }


    //Getters
    /**
     * Gets the index of the first available space within {@link #orderList}. (-1 if full).
     * @return Integer of the first empty index. (-1 if full)
     */
    public int getFirstAvailable()
    {
        //Gets the index of the first available (empty) space in the array (or if full, returns -1)
        for (int i = 0; i < 9; i++)
        {
            if (orderList[i] == null)
            {
                return i;
            }
        }
        return -1;
    }
    
    /**
     * Gets the furniture object of the given index in the {@link #orderList} if valid. (Else, returns null).
     * @param index Index of the item within the array.
     * @return Furniture item of the given index.
     */
    public Furniture get(int index)
    {
        //Make sure the value passed is within the array indexes
        if (isValidIndex(index))
        {
            //Returns the value in the given index. If nothing is there, returns null.
            return orderList[index];
        }
        //If not a valid index, will return null
        return null;
    }
    
    /**
     * Adds an item furniture to the {@link #orderList} at the given index.
     * @param f Furniture object to add.
     * @param index Index within the {@link #orderList} array.
     * @param force If a furniture object already exists there, passing this as true will overwrite it. If this is false, will just skip.
     */
    public void add(Furniture f, int index, boolean force)
    {

        //If force is true, it will overwrite the entry in the array at the index provided (if valid). Else, won't.
        if ((get(index) == null || force == true) && isValidIndex(index))
        {
            orderList[index] = f;
        }
        //Don't do anything if the if statement isn't satisfied.
    }
    /**
     * Deletes (nulls) a furniture object in {@link #orderList} where the given index is. Then shifts everything in the array into order again.
     * @param index Index of item in the list that gets deleted.
     */
    public void del(int index)
    {
        if (isValidIndex(index))
        {
            orderList[index] = null;
        }
        normalise();
    }
    /**
     * Deletes (nulls) everything in the list.
     */
    public void deleteAll()
    {
        for (int i = 0; i < 9; i++)
        {
            orderList[i] = null;
        }
        normalise();
    }
    /**
     * @return Max array size.
     */
    public int getMaxSize()
    {
        return arraySize;
    }
    /**
     * Current array size. (Will always be {@link #arraySize}).
     * @return Int of the size of the array.
     */
    public int getCurrentSize()
    {
        return orderList.length;
    }
    /**
     * Gets the number of free spaces within the {@link #orderList}.
     * @return Int of the number of spaces in the array available.
     */
    public int getFreeSpace()
    {
        int counter = 0;
        for (int i = 0; i < 9; i++)
        {
            if (orderList[i] == null)
            {
                counter++;
            }
        }
        return counter;
    }
    /**
     * Gets the price of all the items within {@link #orderList}.
     * @return A double of the price of all the items.
     */
    public double getPrice()
    {
        double price = 0;
        for (int i = 0; i < 9; i++)
        {
            if (orderList[i] != null)
            {
                price += orderList[i].getPrice();
            }
        }
        return price;
    }
    /**
     * Checks if the given number is a valid index (0 to {@link #arraySize}).
     * @param i Index to check if it's valid.
     * @return True/False based on if it's valid or not.
     */
    public boolean isValidIndex(int i)
    {
        if (i < 9 && i >= 0)
        {
            return true;
        }
        return false;
    }
    /**
     * Shifts all objects within {@link #orderList} so that there are no gaps in between items in the array.
     */
    public void normalise()
    {
        Furniture newOrderList[] = new Furniture[9];
        int arrayCounter = 0;
        for (int i = 0; i < 9; i++)
        {
            if (orderList[i] != null)
            {
                newOrderList[arrayCounter] = orderList[i];
                arrayCounter++;
            }
        }
        orderList = newOrderList;
    }
}
